from .pyup import PyUp
from .__main__ import *

class Pomodoro(PyUp):

    def __init__(self, pyup: PyUp):
        '''
        Some api calls for communication with the Pomodoro record in LifeUp.
        '''
        super().__init__(pyup.host, pyup.port)
        self.host = pyup.host
        self.port = pyup.port

    def add_record(self, start_time: datetime = None, duration: int = None, end_time: datetime = None, reward_tomatoes: bool = None, task_id: int = None, task_gid: int = None, task_name: str = None):
        '''
        Adds new Pomodoro record to LifeUp.


        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if start_time is not None:
            start_time = time_to_epoch(start_time)

        if end_time is not None:
            if duration is not None:
                if start_time is not None and not duration <= start_time - end_time:
                    raise ValueError("param: duration: value must be less than or equal to start_time - end_time if all three params are given.")
                elif duration < 30:
                    raise ValueError("param: duration: value must be greater than or equal to 30.")
                else:
                    duration = duration * 1000
            elif start_time is not None:
                if end_time <= start_time:
                    raise ValueError("param: end_time: value must be greater than start_time.")
                elif end_time - start_time < 30:
                    raise ValueError("param: end_time or start_time: the value of end_time - start_time must be greater than or equal to 30.")
            else:
                raise ValueError("param: end_time: requires a value given to either start_time or duration.")            
            end_time = time_to_epoch(end_time)
        elif duration is not None and duration >= 30:
            duration = duration * 1000
        else:
            raise ValueError("OneOfType: duration, end_time or start_time must have a value.")

        if task_gid or task_id or task_name is None:
            raise ValueError("OneOfType: task_id, task_gid or task_name must have a value.")            
        if task_id is not None and task_id < 1:
            raise ValueError("param: task_id: value must be greater than 0")
        
        if task_gid is not None and task_gid < 1:
            raise ValueError("param: task_gid: value must be greater than 0")

        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/add_pomodoro?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)
            
        return result
    
    def edit_pomodoro(self, edit_item_id: int, start_time: datetime = None, duration: int = None, end_time: datetime = None, reward_tomatoes: bool = None, task_id: int = None, task_gid: int = None, task_name: str = None, ui: bool = None):
        '''
        Edits a record with the given edit_item_id. If not record is found, a new one will be created.


        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if start_time is not None:
            start_time = time_to_epoch(start_time)

        if end_time is not None:
            if duration is not None:
                if start_time is not None and not duration <= start_time - end_time:
                    raise ValueError("param: duration: value must be less than or equal to start_time - end_time if all three params are given.")
                elif duration < 30:
                    raise ValueError("param: duration: value must be greater than or equal to 30.")
                else:
                    duration = duration * 1000
            elif start_time is not None:
                if end_time <= start_time:
                    raise ValueError("param: end_time: value must be greater than start_time.")
                elif end_time - start_time < 30:
                    raise ValueError("param: end_time or start_time: the value of end_time - start_time must be greater than or equal to 30.")
            else:
                raise ValueError("param: end_time: requires a value given to either start_time or duration.")            
            end_time = time_to_epoch(end_time)
        elif duration is not None and duration >= 30:
            duration = duration * 1000
        else:
            raise ValueError("OneOfType: duration, end_time or start_time must have a value.")

        if task_gid or task_id or task_name is None:
            raise ValueError("OneOfType: task_id, task_gid or task_name must have a value.")            
        if task_id is not None and task_id < 1:
            raise ValueError("param: task_id: value must be greater than 0")
        
        if task_gid is not None and task_gid < 1:
            raise ValueError("param: task_gid: value must be greater than 0")

        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/edit_pomodoro?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)
            
        return result


"""
PyUp
=====

an easy to use python library for integrating LifeUp into your projects.
"""
from .general import General
from .task import Task
from .shop import Shop
from .pyup import PyUp
from .atm import ATM
from .pomodoro import Pomodoro

__all__ = ["Task", "General", "PyUp", "Shop", "ATM", "Pomodoro"]
__version__ = "0.1.1"


from requests import Response, Session
from enum import Enum
import re
from datetime import datetime

http = Session()
class Type(Enum):
  COIN = 'coin'
  EXP = 'exp'
  ITEM = 'item'

class CountSetType(Enum):
  ABSOLUTE = 'absolute'
  RELATIVE = 'relative'
        

def call(call,host,port) -> Response:
  
  url = 'http://' + host + ':' + port + '/api?' + call
  return http.get(url=url)

def color_check(hex: str):
  match = re.seach(r'^#(?:[0-9a-fA-F]{3}){1,2}$', hex)

  if match:                      
    return True
  else:
    return False

def time_to_epoch(date:datetime):

  return int(date.timestamp()) * 1000




from .pyup import PyUp
from .__main__ import *

class Shop(PyUp):

    def __init__(self, pyup:PyUp):
        '''
        Some api calls for communicating with the shop in LifeUp.
        '''
        super().__init__(pyup.host, pyup.port)
        self.host = pyup.host
        self.port = pyup.port

    def add_item(self, name: str, desc: str = None, icon: str = None, price: int = None, action_text: str = None, disable_purchase: bool = None, stock_number: int = None, category: int = None, order: int = None, own_number: int = None, unlist: bool = None) -> Response:
        '''
        Adds a new item to the shop.
        
        :param
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if price != None and price < 1 or price > 999999:
            raise ValueError("param: price: must have a value less than 999999 and greater than or equal to 1")
        elif stock_number != None and stock_number < -1 or stock_number > 999999:
            raise ValueError("param: stock_number: must have a value less than 999999 and greater than or equal to -1")
        elif category != None and category < 0:
                raise ValueError("param: category: must have a value greater than or equal to 0")
                
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/add_item?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)

        return result

    def edit_item(self, id: int = None, name: str = None, set_name: str = None, set_desc: str = None, set_icon: str = None, set_price: int = None, set_price_type: CountSetType = None, action_text: str = None, disable_purchase: bool = None, disable_use: bool = None, stock_number: int = None, stock_number_type: CountSetType = None, title_color_string: str = None, category_id: int = None, order: int = None, own_number: int = None, own_number_type: CountSetType = None, unlist: bool = None) -> Response:
        '''
        Edits the specified item in the shop.
        
        :param
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if set_price != None and set_price < 1 or set_price > 999999:
            raise ValueError("param: price: must have a value less than 999999 and greater than or equal to 1")
        elif stock_number != None and stock_number < -1 or stock_number > 999999:
            raise ValueError("param: stock_number: must have a value less than 999999 and greater than or equal to -1")
        elif category_id != None and category_id < 0:
                raise ValueError("param: category: must have a value greater than or equal to 0")
        elif not color_check(title_color_string):
            raise ValueError("param: title_color_string: must be a valid hex color code.")
                
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/item?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)

        return result

    def edit_loot_box(self, id: int = None, name: str = None, sub_id: int = None, sub_name: str = None, set_type: CountSetType = None, amount: int = None, probability: int = None, fixed: bool = None) -> Response:
        '''
        Modify the loot box effect of the specified box item, support adjustment of probability, number of rewards and increase content.
        
        :param
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
                
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        if id or name is None:
            raise ValueError("param: id || name: must have a value.")
        if sub_id or sub_name is None:
            raise ValueError("param: sub_id || sub_name: must have a value.")
        if id is not None and id < 1:
            raise ValueError("param: id: must have a value greater than 0")
        if sub_id is not None and sub_id < 1:
            raise ValueError("param: sub_id: must have a value greater than 0")

        url = 'url=lifeup://api/loot_box?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)

        return result

    def use_item(self, id: int = None, name: str = None, use_times: int = None) -> Response:
        '''
        Modify the loot box effect of the specified box item, support adjustment of probability, number of rewards and increase content.
        
        :param
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
                
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        if id or name is None:
            raise ValueError("param: id || name: must have a value.")
        if id is not None and id < 1:
            raise ValueError("param: id: must have a value greater than 0")
        if use_times is not None and use_times < 1:
            raise ValueError("param: use_times: must have a value greater than 0")

        url = 'url=lifeup://api/use_item?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)

        return result



from .pyup import PyUp
from .__main__ import *

class General(PyUp):

    def __init__(self, pyup: PyUp):
        '''
        Some general api calls for communication with LifeUp.
        '''
        super().__init__(pyup.host, pyup.port)
        self.host = pyup.host
        self.port = pyup.port

    def reward(self, content: str, type: str, number: int, skills: list = None, item_id: int = None, item_name: str = None, silent: bool = None) -> Response:
        '''
        Provide the reward directly. The reason for the reward can be customized.

        :param str content: text that appears on the reward notification.
        :param str type: reward type (must be either 'coin', 'exp' or 'item').
        :param int number: amount of reward given.
        :param (optional) list skills: array of numbers greater than 0. dicates what skills recieve the exp reward.
        :param (optional) int item_id: the numerical id of the item the user recieves as a reward.
        :param (optional) str item_name: the name of the item the user recieves as a reward.
        :param (optional) bool silent: wether the UI notification appears or not.
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if type != 'coin' or 'exp' or 'item':
            raise ValueError("param: content: value must be a value of 'coin', 'exp' or 'item'.")
        if type == 'item' and item_id == 0 or item_name == '':
            raise ValueError("param: type: value 'item' requires item_id and item_name to have values.")
        
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/reward?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]
        result = call(call=url)
        return result

    def toast(self, text: str, type: int = 0, isLong: bool = False) -> Response:
        '''
        Various styles of messages pop up.
        
        :param str text: the message that appears on the prompt.
        :param (optional) int type: a number from 0-6 defining the text style.
        :param (optional) bool isLong: display duration, True = long, False = short.
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if type < 0 or type > 6 or not isinstance(type,int):
            raise ValueError('param: type: value must be a whole number between 0 and 6')
        
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/toast?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]
        
        result = call(call=url)

        return result

    def penality(self, content: str, type: str, number: int, skills: list = None, item_id:int = None, item_name: str = None, silent: bool = None) -> Response:
        '''
        Provide a penalty directly. The reason for the penalty can be customized.
        
        :param str content: text that appears on the reward notification.
        :param str type: reward type (must be either 'coin', 'exp' or 'item').
        :param int number: amount of reward given.
        :param (optional) list skills: array of numbers greater than 0. dicates what skills recieve the exp reward.
        :param (optional) int item_id: the numerical id of the item the user recieves as a reward.
        :param (optional) str item_name: the name of the item the user recieves as a reward.
        :param (optional) bool silent: wether the UI notification appears or not.
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''

        if type == 'item' and item_id == 0 and item_name == '':
            raise ValueError("param: type: value 'item' requires item_id and item_name.")

        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/penality?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url)
        
        return result


class PyUp:
    host: str
    port: str

    def __init__(self,host:str,port:str):
        '''
        Creates the base LifeUp api interaction with a given host and port string.

        :param str host: the ip address of your LifeUp Cloud.
        :param str port: the port address of your LifeUp Cloud.
        '''
        self.host = host
        self.port = port


from .pyup import PyUp
from .__main__ import *

class ATM(PyUp):

    def __init__(self, pyup:PyUp):
        '''
        Some api calls for communicating with the shop in LifeUp.
        '''
        super().__init__(pyup.host, pyup.port)
        self.host = pyup.host
        self.port = pyup.port

    def deposit(self, amount: int):

        params = vars()
        try:
            params.pop('self')
        finally:
            pass
        
        if amount < 1:
            raise ValueError('param: amount: value must be greater than 0')
        
        url = 'url=lifeup://api/deposit?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)

        return result
    
    def withdraw(self, amount: int):

        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        if amount < 1:
            raise ValueError('param: amount: value must be greater than 0')

        url = 'url=lifeup://api/withdraw?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)

        return result


from .pyup import PyUp
from .__main__ import *

class Task(PyUp):

    def __init__(self, pyup:PyUp):
        '''
        Some api calls for communicating with task objects in LifeUp.
        '''
        super().__init__(pyup.host, pyup.port)
        self.host = pyup.host
        self.port = pyup.port

    def add_task(self, todo: str, notes: str = None, coin: int = None, coin_var: int = None, exp: int = None, skills: list = None, category: int = None, frequency: int = None, importance: int = None, difficulty: int = None, item_id: int = None, item_name: str = None, item_amount: int = None, deadline: int = None, color: str = None, background_url: str = None, frozen: bool = None, freeze_until: int = None, coin_penality_factor: float = None, exp_pentalty_factor: float = None, write_feelings: bool = None) -> Response:
        '''
        Creates a task.
        
        :param
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if not color_check(color):
            raise ValueError("param: title_color_string: value must be a valid hex color code.")
        
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/add_task?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)

        return result
        
    def complete_task(self, id: int = None, gid: int = None, name: str = None, ui: bool = None, count: int = None, count_set_type: CountSetType = None, count_force_sum_up: bool = None, reward_factor: float = None) -> Response:
        '''
        Completes a task. Only unfinished tasks will be searched.
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if id or gid or name == None:
            raise ValueError("OneOfType: id, gid or name must have a value.")
        
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/complete?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)
            
        return result

    def give_up_task(self, id: int = None, gid: int = None, name: str = None) -> Response:

        '''
        Gives up a task.
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if id or gid or name == None:
            raise ValueError("OneOfType: id, gid or name must have a value.")
        
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/give_up?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)
            
        return result

    def freeze_task(self, id: int = None, gid: int = None, name: str = None) -> Response:

        '''
        Freezes a task, only for repeating tasks.
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if id or gid or name == None:
            raise ValueError("OneOfType: id, gid or name must have a value.")
        
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/freeze?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)
            
        return result
            
    def unfreeze_task(self, id: int = None, gid: int = None, name: str = None) -> Response:

        '''
        Unfreezes a task, only for repeating tasks.
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if id or gid or name == None:
            raise ValueError("OneOfType: id, gid or name must have a value.")
        
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/unfreeze?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)
            
        return result

    def delete_task(self, id: int = None, gid: int = None, name: str = None) -> Response:

        '''
        Deletes a task.
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        '''
        if id or gid or name == None:
            raise ValueError("OneOfType: id, gid or name must have a value.")
        
        params = vars()
        try:
            params.pop('self')
        finally:
            pass

        url = 'url=lifeup://api/delete_task?'
        for key, value in params.items():
            if value is not None:
                url = url + key + '=' + str(value) + '%26'
            else:
                continue
        url = url[:-3]

        result = call(call=url,host=self.host,port=self.port)
            
        return result
